import os.path
import sys

FLAVORS = ('fetch', 'dump', 'sync', 'install')
MANDATORY_HEADERS = ('Daemon', 'Version')

def dump_config(supported_flavors, path):
    print('conduit {} {{'.format(supported_flavors))
    print('\tpath: "{}";'.format(os.path.abspath(path)))
    print('\ttype: none;') #XXX
    #XXX
    print('}')

def parse_args(argv, supported_flavors):
    if len(argv) < 3:
        raise RuntimeError('Missing conduit argument')
    if argv[1] != 'conduit':
        if argv[1] in ('-config', '--config'):
            dump_config(supported_flavors, argv[0])
            sys.exit(0)
        raise RuntimeError('Missing conduit argument')
    #TODO support "run" instead of "conduit"
    if argv[2].lower() not in FLAVORS:
        raise RuntimeError('Invalid conduit flavor')
    return argv[2].lower()

def headers_from_stdin():
    headers = dict()
    pref_lengths = list()
    prefs = dict()
    while sys.stdin:
        line = sys.stdin.readline().strip()
        if line == '': break
        key, val = line.split(': ', 1)
        if key == 'Preference':
            # Preference data comes after all the headers
            key, length = val.rsplit('/', 1)
            length = int(length)
            pref_lengths.append((key, length))
        else:
            headers[key] = val
    for key, length in pref_lengths:
        prefs[key] = sys.stdin.read(length)
    return headers, prefs

def headers_from_args(argv):
    pass

def conduit_main(**handlers):
    '''
    Call with fetch= dump= sync= set to functions for each flavor you want to support.
    Handler function args will be (input_filename, output_filename, prefs, **headers)
    '''
    try:
        invalid_args = [k for k in handlers.keys() if k not in FLAVORS]
        if invalid_args:
            raise TypeError('Invalid arguments to conduit_main: {}'.format(
                ', '.join(invalid_args)))
        selected_flavor = parse_args(sys.argv, handlers.keys())
        if selected_flavor not in handlers:
            raise RuntimeError('This conduit does not support {}'.format(selected_flavor))
        headers, prefs = headers_from_stdin()
        if 'SPCPipe' in headers:
            raise NotImplementedError('Python library does not support SPC')
        # Prune some headers so only conduit-specific ones should remain
        input_db = headers.pop('InputDB', None)
        output_db = headers.pop('OutputDB', None)
        headers.pop('Daemon')
        headers.pop('Version')
    except Exception as e:
        # Put error code on stdout so coldsync sees it
        print('400 {}'.format(e))
        raise
    try:
        result_msg = handlers[selected_flavor](input_db, output_db, prefs, **headers)
    except Exception as e:
        # Put error code on stdout so coldsync sees it
        print('500 {}'.format(e))
        raise
    print('202 {}'.format(result_msg or 'Success'))
