#!/usr/bin/env python3
import re, sys
from datetime import date
from coldsync import conduit_main
from palm import ToDoDB

def _parse_todotxt(line):
    todo_regex = re.compile(r"""
    (?P<completed>x\ )?
    (\((?P<priority>[A-Z])\)\ )?
    (?(completed)((?P<completion_date>\d{4}-\d\d-\d\d)\ )?)
    ((?P<creation_date>\d{4}-\d\d-\d\d)\ )?
    (?P<description>.*)
    """, re.X)
    match = todo_regex.fullmatch(line)
    out = match.groupdict()
    out['completed'] = bool(out['completed'])
    if out['priority']:
        out['priority'] = ord(out['priority']) - ord('A') + 1
    if out['completion_date']:
        out['completion_date'] = date.fromisoformat(out['completion_date'])
    if out['creation_date']:
        out['creation_date'] = date.fromisoformat(out['creation_date'])
    return out

def _dump_todotxt(rec):
    if rec['completed']:
        s = 'x '
        if rec['completion_date'] and rec['creation_date']:
            s += '{} {} '.format(rec['completion_date'], rec['creation_date'])
    else:
        s = ''
        if rec['priority']:
            s += '({}) '.format(chr(ord('A') + rec['priority'] - 1))
        if rec['creation_date']:
            s += '{} '.format(rec['creation_date'])
    s += rec['description']
    return s

def pdb_to_todotxt(in_fname, _, prefs, File, Archive=None, **headers):
    with open(in_fname, 'rb') as f:
        db = ToDoDB(f)
    with open(File, 'r') as f:
        unmatched = list(_parse_todotxt(line.strip()) for line in f)
    matched = []
    # match palm records to existing records & update
    for r in db.recordData:
        pass
    # dump all in todotxt format
    out = list(_dump_todotxt(r)+'\n' for r in (matched + unmatched))
    with open(File, 'w') as f:
        f.writelines(out)

if __name__ == '__main__':
    conduit_main(dump=pdb_to_todotxt)
